@@ -54,12 +54,12 @@ EVENT_CREATE, EVENT_UPDATE, EVENT_DELETE = ("create", "update", "delete")
 class ResourcesDefinition(object):
 
     MANDATORY_FIELDS = {'resource_type': six.string_types,
-                        'metrics': list}
+                        'metrics': (dict, list)}
 
     MANDATORY_EVENT_FIELDS = {'id': six.string_types}
 
-    def __init__(self, definition_cfg, default_archive_policy, plugin_manager):
-        self._default_archive_policy = default_archive_policy
+    def __init__(self, definition_cfg, archive_policy_default,
+                 archive_policy_override, plugin_manager):
         self.cfg = definition_cfg
 
         self._check_required_and_types(self.MANDATORY_FIELDS, self.cfg)
@@ -79,24 +79,44 @@ class ResourcesDefinition(object):
                 name, attr_cfg, plugin_manager)
 
         self.metrics = {}
-        for t in self.cfg['metrics']:
-            archive_policy = self.cfg.get('archive_policy',
-                                          self._default_archive_policy)
-            if archive_policy is None:
-                self.metrics[t] = {}
-            else:
-                self.metrics[t] = dict(archive_policy_name=archive_policy)
+
+        # NOTE(sileht): Convert old list to new dict format
+        if isinstance(self.cfg['metrics'], list):
+            values = [None] * len(self.cfg['metrics'])
+            self.cfg['metrics'] = dict(zip(self.cfg['metrics'], values))
+
+        for m, extra in self.cfg['metrics'].items():
+            if not extra:
+                extra = {}
+
+            if not extra.get("archive_policy_name"):
+                extra["archive_policy_name"] = archive_policy_default
+
+            if archive_policy_override:
+                extra["archive_policy_name"] = archive_policy_override
+
+            # NOTE(sileht): For backward compat, this is after the override to
+            # preserve the wierd previous behavior. We don't really care as we
+            # deprecate it.
+            if 'archive_policy' in self.cfg:
+                LOG.warning("archive_policy '%s' for a resource-type (%s) is "
+                            "deprecated, set it for each metric instead.",
+                            self.cfg["archive_policy"],
+                            self.cfg["resource_type"])
+                extra["archive_policy_name"] = self.cfg['archive_policy']
+
+            self.metrics[m] = extra
 
     @staticmethod
     def _check_required_and_types(expected, definition):
-        for field, field_type in expected.items():
+        for field, field_types in expected.items():
             if field not in definition:
                 raise declarative.ResourceDefinitionException(
                     _("Required field %s not specified") % field, definition)
-            if not isinstance(definition[field], field_type):
+            if not isinstance(definition[field], field_types):
                 raise declarative.ResourceDefinitionException(
                     _("Required field %(field)s should be a %(type)s") %
-                    {'field': field, 'type': field_type}, definition)
+                    {'field': field, 'type': field_types}, definition)
 
     @staticmethod
     def _ensure_list(value):
@@ -178,23 +198,19 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
         # TODO(jd) allow to override Gnocchi endpoint via the host in the URL
         options = urlparse.parse_qs(parsed_url.query)
 
-        self.filter_project = options.get(
-            'filter_project',
-            [conf.dispatcher_gnocchi.filter_project])[-1]
+        self.filter_project = options.get('filter_project', ['service'])[-1]
 
         resources_definition_file = options.get(
-            'resources_definition_file',
-            [conf.dispatcher_gnocchi.resources_definition_file])[-1]
-        archive_policy = options.get(
-            'archive_policy',
-            [conf.dispatcher_gnocchi.archive_policy])[-1]
-        self.resources_definition = self._load_resources_definitions(
-            conf, archive_policy, resources_definition_file)
+            'resources_definition_file', ['gnocchi_resources.yaml'])[-1]
+
+        archive_policy_override = options.get('archive_policy', [None])[-1]
+        self.resources_definition, self.archive_policies_definition = (
+            self._load_definitions(conf, archive_policy_override,
+                                   resources_definition_file))
         self.metric_map = dict((metric, rd) for rd in self.resources_definition
                                for metric in rd.metrics)
 
-        timeout = options.get('timeout',
-                              [conf.dispatcher_gnocchi.request_timeout])[-1]
+        timeout = options.get('timeout', [6.05])[-1]
         self._ks_client = keystone_client.get_client(conf)
 
         self.cache = None
@@ -224,25 +240,40 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
         self._already_logged_event_types = set()
         self._already_logged_metric_names = set()
 
+        self._already_configured_archive_policies = False
+
     @staticmethod
-    def _load_resources_definitions(conf, archive_policy,
-                                    resources_definition_file):
+    def _load_definitions(conf, archive_policy_override,
+                          resources_definition_file):
         plugin_manager = extension.ExtensionManager(
             namespace='ceilometer.event.trait_plugin')
         data = declarative.load_definitions(
             conf, {}, resources_definition_file,
             pkg_resources.resource_filename(__name__,
                                             "data/gnocchi_resources.yaml"))
+
+        archive_policy_default = data.get("archive_policy_default", "low")
         resource_defs = []
         for resource in data.get('resources', []):
             try:
                 resource_defs.append(ResourcesDefinition(
                     resource,
-                    archive_policy, plugin_manager))
+                    archive_policy_default,
+                    archive_policy_override,
+                    plugin_manager))
             except Exception as exc:
                 LOG.error("Failed to load resource due to error %s" %
                           exc)
-        return resource_defs
+        return resource_defs, data.get("archive_policies", [])
+
+    def ensures_archives_policies(self):
+        if not self._already_configured_archive_policies:
+            for ap in self.archive_policies_definition:
+                try:
+                    self._gnocchi.archive_policy.get(ap["name"])
+                except gnocchi_exc.ArchivePolicyNotFound:
+                    self._gnocchi.archive_policy.create(ap)
+            self._already_configured_archive_policies = True
 
     @property
     def gnocchi_project_id(self):
@@ -290,6 +321,8 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
                 return rd, operation
 
     def publish_samples(self, data):
+        self.ensures_archives_policies()
+
         # NOTE(sileht): skip sample generated by gnocchi itself
         data = [s for s in data if not self._is_gnocchi_activity(s)]
         data.sort(key=operator.attrgetter('resource_id'))
@@ -317,17 +350,20 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
                         'resource_type': rd.cfg['resource_type'],
                         'resource': {"id": resource_id,
                                      "user_id": sample.user_id,
-                                     "project_id": sample.project_id,
-                                     "metrics": rd.metrics}}
+                                     "project_id": sample.project_id}}
 
                 gnocchi_data[resource_id].setdefault(
                     "resource_extra", {}).update(rd.sample_attributes(sample))
                 measures.setdefault(resource_id, {}).setdefault(
-                    metric_name, []).append({'timestamp': sample.timestamp,
-                                             'value': sample.volume})
-                # TODO(gordc): unit should really be part of metric definition
-                gnocchi_data[resource_id]['resource']['metrics'][
-                    metric_name]['unit'] = sample.unit
+                    metric_name,
+                    {"measures": [],
+                     "archive_policy_name":
+                     rd.metrics[metric_name]["archive_policy_name"],
+                     "unit": sample.unit}
+                )["measures"].append(
+                    {'timestamp': sample.timestamp,
+                     'value': sample.volume}
+                )
 
         try:
             self.batch_measures(measures, gnocchi_data)
@@ -343,8 +379,8 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
             if not resource_extra:
                 continue
             try:
-                self._if_not_cached("update", resource_type, resource,
-                                    self._update_resource, resource_extra)
+                self._if_not_cached(resource_type, resource['id'],
+                                    resource_extra)
             except gnocchi_exc.ClientException as e:
                 LOG.error(six.text_type(e))
             except Exception as e:
@@ -375,8 +411,7 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
             for resource_type, resource, resource_extra in resources:
                 try:
                     resource.update(resource_extra)
-                    self._if_not_cached("create", resource_type, resource,
-                                        self._create_resource)
+                    self._create_resource(resource_type, resource)
                 except gnocchi_exc.ResourceAlreadyExists:
                     # NOTE(sileht): resource created in the meantime
                     pass
@@ -387,6 +422,10 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
                     # and we can't patch it later
                     del measures[resource['id']]
                     del resource_infos[resource['id']]
+                else:
+                    if self.cache and resource_extra:
+                        self.cache.set(resource['id'],
+                                       self._hash_resource(resource_extra))
 
             # NOTE(sileht): we have created missing resources/metrics,
             # now retry to post measures
@@ -395,56 +434,46 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
 
         LOG.debug(
             "%d measures posted against %d metrics through %d resources",
-            sum(len(m) for rid in measures for m in measures[rid].values()),
+            sum(len(m["measures"])
+                for rid in measures
+                for m in measures[rid].values()),
             sum(len(m) for m in measures.values()), len(resource_infos))
 
     def _create_resource(self, resource_type, resource):
         self._gnocchi.resource.create(resource_type, resource)
         LOG.debug('Resource %s created', resource["id"])
 
-    def _update_resource(self, resource_type, resource, resource_extra):
-        self._gnocchi.resource.update(resource_type,
-                                      resource["id"],
-                                      resource_extra)
-        LOG.debug('Resource %s updated', resource["id"])
+    def _update_resource(self, resource_type, res_id, resource_extra):
+        self._gnocchi.resource.update(resource_type, res_id, resource_extra)
+        LOG.debug('Resource %s updated', res_id)
 
-    def _if_not_cached(self, operation, resource_type, resource, method,
-                       *args, **kwargs):
+    def _if_not_cached(self, resource_type, res_id, resource_extra):
         if self.cache:
-            cache_key = resource['id']
-            attribute_hash = self._check_resource_cache(cache_key, resource)
-            hit = False
-            if attribute_hash:
-                with self._gnocchi_resource_lock[cache_key]:
+            attribute_hash = self._hash_resource(resource_extra)
+            if self._resource_cache_diff(res_id, attribute_hash):
+                with self._gnocchi_resource_lock[res_id]:
                     # NOTE(luogangyi): there is a possibility that the
                     # resource was already built in cache by another
                     # ceilometer-notification-agent when we get the lock here.
-                    attribute_hash = self._check_resource_cache(cache_key,
-                                                                resource)
-                    if attribute_hash:
-                        method(resource_type, resource, *args, **kwargs)
-                        self.cache.set(cache_key, attribute_hash)
+                    if self._resource_cache_diff(res_id, attribute_hash):
+                        self._update_resource(resource_type, res_id,
+                                              resource_extra)
+                        self.cache.set(res_id, attribute_hash)
                     else:
-                        hit = True
-                        LOG.debug('resource cache recheck hit for '
-                                  '%s %s', operation, cache_key)
-                self._gnocchi_resource_lock.pop(cache_key, None)
+                        LOG.debug('Resource cache hit for %s', res_id)
+                self._gnocchi_resource_lock.pop(res_id, None)
             else:
-                hit = True
-                LOG.debug('Resource cache hit for %s %s', operation, cache_key)
-            if hit and operation == "create":
-                raise gnocchi_exc.ResourceAlreadyExists()
+                LOG.debug('Resource cache hit for %s', res_id)
         else:
-            method(resource_type, resource, *args, **kwargs)
+            self._update_resource(resource_type, res_id, resource_extra)
 
-    def _check_resource_cache(self, key, resource_data):
+    @staticmethod
+    def _hash_resource(resource):
+        return hash(tuple(i for i in resource.items() if i[0] != 'metrics'))
+
+    def _resource_cache_diff(self, key, attribute_hash):
         cached_hash = self.cache.get(key)
-        attribute_hash = hash(tuple(i for i in resource_data.items()
-                                    if i[0] != 'metrics'))
-        if not cached_hash or cached_hash != attribute_hash:
-            return attribute_hash
-        else:
-            return None
+        return not cached_hash or cached_hash != attribute_hash
 
     def publish_events(self, events):
         for event in events:
