@@ -27,38 +27,28 @@ from oslo_utils import timeutils
 import six
 from stevedore import extension
 
-from ceilometer import pipeline
+from ceilometer.pipeline import base as pipe_base
+from ceilometer.pipeline import sample as pipeline
 from ceilometer import publisher
 from ceilometer.publisher import test as test_publisher
 from ceilometer import sample
 from ceilometer import service
+from ceilometer.tests import base
 from ceilometer import transformer
 from ceilometer.transformer import accumulator
 from ceilometer.transformer import arithmetic
 from ceilometer.transformer import conversions
 
-from ceilometer.tests import base
-
 
 @six.add_metaclass(abc.ABCMeta)
 class BasePipelineTestCase(base.BaseTestCase):
-    @staticmethod
-    def fake_tem_init():
-        """Fake a transformerManager for pipeline.
 
-        The faked entry point setting is below:
-        update: TransformerClass
-        except: TransformerClassException
-        drop:   TransformerClassDrop
-        """
-        pass
-
-    def fake_tem_get_ext(self, name):
+    def fake_transform_manager(self):
         class_name_ext = {
             'update': self.TransformerClass,
             'except': self.TransformerClassException,
             'drop': self.TransformerClassDrop,
-            'cache': accumulator.TransformerAccumulator,
+            'accumulator': accumulator.TransformerAccumulator,
             'aggregator': conversions.AggregatorTransformer,
             'unit_conversion': conversions.ScalingTransformer,
             'rate_of_change': conversions.RateOfChangeTransformer,
@@ -66,13 +56,9 @@ class BasePipelineTestCase(base.BaseTestCase):
             'delta': conversions.DeltaTransformer,
         }
 
-        if name in class_name_ext:
-            return extension.Extension(name, None,
-                                       class_name_ext[name],
-                                       None,
-                                       )
-
-        raise KeyError(name)
+        return extension.ExtensionManager.make_test_instance([
+            extension.Extension(name, None, transformer, None)
+            for name, transformer in class_name_ext.items()])
 
     def get_publisher(self, conf, url, namespace=''):
         fake_drivers = {'test://': test_publisher.TestPublisher,
@@ -149,16 +135,15 @@ class BasePipelineTestCase(base.BaseTestCase):
 
         self.useFixture(fixtures.MockPatchObject(
             publisher, 'get_publisher', side_effect=self.get_publisher))
-
-        self.transformer_manager = mock.MagicMock()
-        self.transformer_manager.__getitem__.side_effect = \
-            self.fake_tem_get_ext
+        self.useFixture(fixtures.MockPatchObject(
+            pipeline.SamplePipelineManager, 'get_transform_manager',
+            side_effect=self.fake_transform_manager))
 
         self._setup_pipeline_cfg()
 
         self._reraise_exception = True
         self.useFixture(fixtures.MockPatch(
-            'ceilometer.pipeline.LOG.exception',
+            'ceilometer.pipeline.base.LOG.exception',
             side_effect=self._handle_reraise_exception))
 
     def _handle_reraise_exception(self, *args, **kwargs):
@@ -193,12 +178,14 @@ class BasePipelineTestCase(base.BaseTestCase):
     def _unset_pipeline_cfg(self, field):
         """Clear an existing field in the pipeline config."""
 
+    def _build_and_set_new_pipeline(self):
+        name = self.cfg2file(self.pipeline_cfg)
+        self.CONF.set_override('pipeline_cfg_file', name)
+
     def _exception_create_pipelinemanager(self):
-        self.assertRaises(pipeline.PipelineException,
-                          pipeline.PipelineManager,
-                          self.CONF,
-                          self.cfg2file(self.pipeline_cfg),
-                          self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        self.assertRaises(pipe_base.PipelineException,
+                          pipeline.SamplePipelineManager, self.CONF)
 
     def test_no_meters(self):
         self._unset_pipeline_cfg('meters')
@@ -206,9 +193,8 @@ class BasePipelineTestCase(base.BaseTestCase):
 
     def test_no_transformers(self):
         self._unset_pipeline_cfg('transformers')
-        pipeline.PipelineManager(self.CONF,
-                                 self.cfg2file(self.pipeline_cfg),
-                                 self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline.SamplePipelineManager(self.CONF)
 
     def test_no_name(self):
         self._unset_pipeline_cfg('name')
@@ -246,9 +232,8 @@ class BasePipelineTestCase(base.BaseTestCase):
         self._exception_create_pipelinemanager()
 
     def test_publisher_transformer_invoked(self):
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
 
@@ -262,10 +247,8 @@ class BasePipelineTestCase(base.BaseTestCase):
     def test_multiple_included_counters(self):
         counter_cfg = ['a', 'b']
         self._set_pipeline_cfg('meters', counter_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
-
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
 
@@ -292,12 +275,11 @@ class BasePipelineTestCase(base.BaseTestCase):
         self.assertEqual('a_update', getattr(publisher.samples[0], "name"))
         self.assertEqual('b_update', getattr(publisher.samples[1], "name"))
 
-    @mock.patch('ceilometer.pipeline.LOG')
+    @mock.patch('ceilometer.pipeline.sample.LOG')
     def test_none_volume_counter(self, LOG):
         self._set_pipeline_cfg('meters', ['empty_volume'])
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         publisher = pipeline_manager.pipelines[0].publishers[0]
 
         test_s = sample.Sample(
@@ -326,12 +308,11 @@ class BasePipelineTestCase(base.BaseTestCase):
 
         self.assertEqual(0, len(publisher.samples))
 
-    @mock.patch('ceilometer.pipeline.LOG')
+    @mock.patch('ceilometer.pipeline.sample.LOG')
     def test_fake_volume_counter(self, LOG):
         self._set_pipeline_cfg('meters', ['fake_volume'])
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         publisher = pipeline_manager.pipelines[0].publishers[0]
 
         test_s = sample.Sample(
@@ -363,9 +344,8 @@ class BasePipelineTestCase(base.BaseTestCase):
     def test_counter_dont_match(self):
         counter_cfg = ['nomatch']
         self._set_pipeline_cfg('meters', counter_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
 
@@ -376,9 +356,8 @@ class BasePipelineTestCase(base.BaseTestCase):
     def test_wildcard_counter(self):
         counter_cfg = ['*']
         self._set_pipeline_cfg('meters', counter_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
 
@@ -390,17 +369,16 @@ class BasePipelineTestCase(base.BaseTestCase):
     def test_wildcard_excluded_counters(self):
         counter_cfg = ['*', '!a']
         self._set_pipeline_cfg('meters', counter_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
-        self.assertFalse(pipeline_manager.pipelines[0].support_meter('a'))
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
+        pipe = pipeline_manager.pipelines[0]
+        self.assertFalse(pipe.source.support_meter('a'))
 
     def test_wildcard_excluded_counters_not_excluded(self):
         counter_cfg = ['*', '!b']
         self._set_pipeline_cfg('meters', counter_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
         publisher = pipeline_manager.pipelines[0].publishers[0]
@@ -411,9 +389,8 @@ class BasePipelineTestCase(base.BaseTestCase):
     def test_all_excluded_counters_not_excluded(self):
         counter_cfg = ['!b', '!c']
         self._set_pipeline_cfg('meters', counter_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
 
@@ -427,53 +404,46 @@ class BasePipelineTestCase(base.BaseTestCase):
     def test_all_excluded_counters_is_excluded(self):
         counter_cfg = ['!a', '!c']
         self._set_pipeline_cfg('meters', counter_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
-        self.assertFalse(pipeline_manager.pipelines[0].support_meter('a'))
-        self.assertTrue(pipeline_manager.pipelines[0].support_meter('b'))
-        self.assertFalse(pipeline_manager.pipelines[0].support_meter('c'))
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
+        pipe = pipeline_manager.pipelines[0]
+        self.assertFalse(pipe.source.support_meter('a'))
+        self.assertTrue(pipe.source.support_meter('b'))
+        self.assertFalse(pipe.source.support_meter('c'))
 
     def test_wildcard_and_excluded_wildcard_counters(self):
         counter_cfg = ['*', '!disk.*']
         self._set_pipeline_cfg('meters', counter_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
-        self.assertFalse(pipeline_manager.pipelines[0].
-                         support_meter('disk.read.bytes'))
-        self.assertTrue(pipeline_manager.pipelines[0].support_meter('cpu'))
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
+        pipe = pipeline_manager.pipelines[0]
+        self.assertFalse(pipe.source.support_meter('disk.read.bytes'))
+        self.assertTrue(pipe.source.support_meter('cpu'))
 
     def test_included_counter_and_wildcard_counters(self):
         counter_cfg = ['cpu', 'disk.*']
         self._set_pipeline_cfg('meters', counter_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
-        self.assertTrue(pipeline_manager.pipelines[0].
-                        support_meter('disk.read.bytes'))
-        self.assertTrue(pipeline_manager.pipelines[0].support_meter('cpu'))
-        self.assertFalse(pipeline_manager.pipelines[0].
-                         support_meter('instance'))
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
+        pipe = pipeline_manager.pipelines[0]
+        self.assertTrue(pipe.source.support_meter('disk.read.bytes'))
+        self.assertTrue(pipe.source.support_meter('cpu'))
+        self.assertFalse(pipe.source.support_meter('instance'))
 
     def test_excluded_counter_and_excluded_wildcard_counters(self):
         counter_cfg = ['!cpu', '!disk.*']
         self._set_pipeline_cfg('meters', counter_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
-        self.assertFalse(pipeline_manager.pipelines[0].
-                         support_meter('disk.read.bytes'))
-        self.assertFalse(pipeline_manager.pipelines[0].support_meter('cpu'))
-        self.assertTrue(pipeline_manager.pipelines[0].
-                        support_meter('instance'))
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
+        pipe = pipeline_manager.pipelines[0]
+        self.assertFalse(pipe.source.support_meter('disk.read.bytes'))
+        self.assertFalse(pipe.source.support_meter('cpu'))
+        self.assertTrue(pipe.source.support_meter('instance'))
 
     def test_multiple_pipeline(self):
         self._augment_pipeline_cfg()
-
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
 
@@ -509,9 +479,8 @@ class BasePipelineTestCase(base.BaseTestCase):
     def test_multiple_pipeline_exception(self):
         self._reraise_exception = False
         self._break_pipeline_cfg()
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
 
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
@@ -543,9 +512,8 @@ class BasePipelineTestCase(base.BaseTestCase):
 
     def test_none_transformer_pipeline(self):
         self._set_pipeline_cfg('transformers', None)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
         publisher = pipeline_manager.pipelines[0].publishers[0]
@@ -555,9 +523,8 @@ class BasePipelineTestCase(base.BaseTestCase):
 
     def test_empty_transformer_pipeline(self):
         self._set_pipeline_cfg('transformers', [])
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
         publisher = pipeline_manager.pipelines[0].publishers[0]
@@ -577,9 +544,8 @@ class BasePipelineTestCase(base.BaseTestCase):
             },
         ]
         self._set_pipeline_cfg('transformers', transformer_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
 
@@ -612,9 +578,8 @@ class BasePipelineTestCase(base.BaseTestCase):
             },
         ]
         self._set_pipeline_cfg('transformers', transformer_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
 
@@ -651,9 +616,8 @@ class BasePipelineTestCase(base.BaseTestCase):
             },
         ]
         self._set_pipeline_cfg('transformers', transformer_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
 
@@ -669,9 +633,8 @@ class BasePipelineTestCase(base.BaseTestCase):
 
     def test_multiple_publisher(self):
         self._set_pipeline_cfg('publishers', ['test://', 'new://'])
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
 
@@ -687,9 +650,8 @@ class BasePipelineTestCase(base.BaseTestCase):
     def test_multiple_publisher_isolation(self):
         self._reraise_exception = False
         self._set_pipeline_cfg('publishers', ['except://', 'new://'])
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter])
 
@@ -700,9 +662,8 @@ class BasePipelineTestCase(base.BaseTestCase):
 
     def test_multiple_counter_pipeline(self):
         self._set_pipeline_cfg('meters', ['a', 'b'])
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter,
                sample.Sample(
@@ -726,7 +687,7 @@ class BasePipelineTestCase(base.BaseTestCase):
         CACHE_SIZE = 10
         extra_transformer_cfg = [
             {
-                'name': 'cache',
+                'name': 'accumulator',
                 'parameters': {
                     'size': CACHE_SIZE,
                 }
@@ -740,9 +701,8 @@ class BasePipelineTestCase(base.BaseTestCase):
             },
         ]
         self._extend_pipeline_cfg('transformers', extra_transformer_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
         pipe.publish_data(self.test_counter)
@@ -763,7 +723,7 @@ class BasePipelineTestCase(base.BaseTestCase):
         CACHE_SIZE = 3
         extra_transformer_cfg = [
             {
-                'name': 'cache',
+                'name': 'accumulator',
                 'parameters': {
                     'size': CACHE_SIZE
                 }
@@ -778,9 +738,8 @@ class BasePipelineTestCase(base.BaseTestCase):
         ]
         self._extend_pipeline_cfg('transformers', extra_transformer_cfg)
         self._set_pipeline_cfg('meters', ['a', 'b'])
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         with pipeline_manager.publisher() as p:
             p([self.test_counter,
                sample.Sample(
@@ -809,13 +768,12 @@ class BasePipelineTestCase(base.BaseTestCase):
 
     def test_flush_pipeline_cache_before_publisher(self):
         extra_transformer_cfg = [{
-            'name': 'cache',
+            'name': 'accumulator',
             'parameters': {}
         }]
         self._extend_pipeline_cfg('transformers', extra_transformer_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
         publisher = pipe.publishers[0]
@@ -854,9 +812,8 @@ class BasePipelineTestCase(base.BaseTestCase):
                 resource_metadata={}
             ),
         ]
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
         pipe.publish_data(counters)
@@ -911,10 +868,8 @@ class BasePipelineTestCase(base.BaseTestCase):
                 resource_metadata={}
             ),
         ]
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg),
-            self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
         pipe.publish_data(counters)
@@ -949,7 +904,7 @@ class BasePipelineTestCase(base.BaseTestCase):
         ]
         self._set_pipeline_cfg('transformers', transformer_cfg)
         self._set_pipeline_cfg('meters', ['cpu'])
-        now = timeutils.utcnow()
+        now = datetime.datetime.utcnow()
         later = now + datetime.timedelta(minutes=offset)
         um = {'autoscaling_weight': weight} if weight else {}
         counters = [
@@ -1002,9 +957,8 @@ class BasePipelineTestCase(base.BaseTestCase):
                                    'user_metadata': um},
             ),
         ]
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
         pipe.publish_data(counters)
@@ -1073,7 +1027,7 @@ class BasePipelineTestCase(base.BaseTestCase):
         ]
         self._set_pipeline_cfg('transformers', transformer_cfg)
         self._set_pipeline_cfg('meters', ['cpu'])
-        now = timeutils.utcnow()
+        now = datetime.datetime.utcnow()
         counters = [
             sample.Sample(
                 name='cpu',
@@ -1087,9 +1041,8 @@ class BasePipelineTestCase(base.BaseTestCase):
                 resource_metadata={'cpu_number': 4}
             ),
         ]
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
         pipe.publish_data(counters)
@@ -1114,12 +1067,11 @@ class BasePipelineTestCase(base.BaseTestCase):
         ]
         self._set_pipeline_cfg('transformers', transformer_cfg)
         self._set_pipeline_cfg('meters', ['cpu'])
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
-        now = timeutils.utcnow()
+        now = datetime.datetime.utcnow()
         now_time = monotonic.monotonic()
         # Simulate a laggy poller
         later = now + datetime.timedelta(seconds=12345)
@@ -1176,12 +1128,11 @@ class BasePipelineTestCase(base.BaseTestCase):
         ]
         self._set_pipeline_cfg('transformers', transformer_cfg)
         self._set_pipeline_cfg('meters', ['cpu'])
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
-        now = timeutils.utcnow()
+        now = datetime.datetime.utcnow()
         later = now + datetime.timedelta(seconds=10)
         rounding = 12345
 
@@ -1234,12 +1185,11 @@ class BasePipelineTestCase(base.BaseTestCase):
         ]
         self._set_pipeline_cfg('transformers', transformer_cfg)
         self._set_pipeline_cfg('meters', ['cpu'])
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
-        now = timeutils.utcnow()
+        now = datetime.datetime.utcnow()
         earlier = now - datetime.timedelta(seconds=10)
         later = now + datetime.timedelta(seconds=10)
 
@@ -1293,7 +1243,7 @@ class BasePipelineTestCase(base.BaseTestCase):
         )
 
     def _do_test_rate_of_change_mapping(self, pipe, meters, units):
-        now = timeutils.utcnow()
+        now = datetime.datetime.utcnow()
         base = 1000
         offset = 7
         rate = 42
@@ -1356,9 +1306,8 @@ class BasePipelineTestCase(base.BaseTestCase):
         self._set_pipeline_cfg('transformers', transformer_cfg)
         self._set_pipeline_cfg('meters', ['disk.read.bytes',
                                           'disk.write.requests'])
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
         meters = ('disk.read.bytes', 'disk.write.requests')
         units = ('B', 'request')
@@ -1441,10 +1390,8 @@ class BasePipelineTestCase(base.BaseTestCase):
                 resource_metadata={'version': '3.0'}
             ),
         ]
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg),
-            self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
         pipe.publish_data(counters)
@@ -1478,9 +1425,8 @@ class BasePipelineTestCase(base.BaseTestCase):
                     timestamp=timeutils.utcnow().isoformat(),
                     resource_metadata={'version': '1.0'}
                 ))
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
         pipe.publish_data(counters)
@@ -1674,9 +1620,8 @@ class BasePipelineTestCase(base.BaseTestCase):
                 resource_metadata={'version': '2.0'}
             )
         ]
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
         pipe.publish_data([counters[0]])
@@ -1689,8 +1634,10 @@ class BasePipelineTestCase(base.BaseTestCase):
         publisher = pipe.publishers[0]
         self.assertEqual(2, len(publisher.samples))
 
-    def test_aggregator_timed_flush(self):
-        timeutils.set_time_override()
+    @mock.patch.object(timeutils, 'utcnow')
+    def test_aggregator_timed_flush(self, mock_utcnow):
+        now = datetime.datetime.utcnow()
+        mock_utcnow.return_value = now
         transformer_cfg = [
             {
                 'name': 'aggregator',
@@ -1712,9 +1659,8 @@ class BasePipelineTestCase(base.BaseTestCase):
                 resource_metadata={'version': '1.0'}
             ),
         ]
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
         pipe.publish_data(counters)
@@ -1722,7 +1668,7 @@ class BasePipelineTestCase(base.BaseTestCase):
         publisher = pipeline_manager.pipelines[0].publishers[0]
         self.assertEqual(0, len(publisher.samples))
 
-        timeutils.advance_time_seconds(120)
+        mock_utcnow.return_value = now + datetime.timedelta(seconds=120)
         pipe.flush()
         publisher = pipeline_manager.pipelines[0].publishers[0]
         self.assertEqual(1, len(publisher.samples))
@@ -1760,9 +1706,8 @@ class BasePipelineTestCase(base.BaseTestCase):
                 resource_metadata={'version': '2.0'}
             )
         ]
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
         pipe.publish_data([counters[0]])
@@ -1907,9 +1852,8 @@ class BasePipelineTestCase(base.BaseTestCase):
                     timestamp=timeutils.utcnow().isoformat(),
                     resource_metadata=s.get('metadata')
                 ))
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
         for s in counters:
             pipe.publish_data(s)
@@ -2027,9 +1971,8 @@ class BasePipelineTestCase(base.BaseTestCase):
             timestamp=timeutils.utcnow().isoformat(),
             resource_metadata=None
         )
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
         pipe.publish_data([counter])
@@ -2048,8 +1991,10 @@ class BasePipelineTestCase(base.BaseTestCase):
         self.assertEqual(2, len(publisher.samples))
         self.assertEqual(2050.0, publisher.samples[1].volume)
 
-    def test_aggregator_timed_flush_no_matching_samples(self):
-        timeutils.set_time_override()
+    @mock.patch.object(timeutils, 'utcnow')
+    def test_aggregator_timed_flush_no_matching_samples(self, mock_utcnow):
+        now = datetime.datetime.utcnow()
+        mock_utcnow.return_value = now
         transformer_cfg = [
             {
                 'name': 'aggregator',
@@ -2058,10 +2003,9 @@ class BasePipelineTestCase(base.BaseTestCase):
         ]
         self._set_pipeline_cfg('transformers', transformer_cfg)
         self._set_pipeline_cfg('meters', ['unrelated-sample'])
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
-        timeutils.advance_time_seconds(200)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
+        mock_utcnow.return_value = now + datetime.timedelta(seconds=200)
         pipe = pipeline_manager.pipelines[0]
         pipe.flush()
         publisher = pipeline_manager.pipelines[0].publishers[0]
@@ -2079,9 +2023,8 @@ class BasePipelineTestCase(base.BaseTestCase):
         ]
         self._set_pipeline_cfg('transformers', transformer_cfg)
         self._set_pipeline_cfg('meters', ['cpu'])
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         pipe = pipeline_manager.pipelines[0]
 
         pipe.publish_data(data)
@@ -2250,12 +2193,10 @@ class BasePipelineTestCase(base.BaseTestCase):
             },
         ]
         self._set_pipeline_cfg('transformers', transformer_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         self.assertEqual(set(['resource_id', 'counter_name']),
-                         set(pipeline.get_pipeline_grouping_key(
-                             pipeline_manager.pipelines[0])))
+                         set(pipeline_manager.pipelines[0].get_grouping_key()))
 
     def test_get_pipeline_duplicate_grouping_key(self):
         transformer_cfg = [
@@ -2269,9 +2210,7 @@ class BasePipelineTestCase(base.BaseTestCase):
             },
         ]
         self._set_pipeline_cfg('transformers', transformer_cfg)
-        pipeline_manager = pipeline.PipelineManager(
-            self.CONF,
-            self.cfg2file(self.pipeline_cfg), self.transformer_manager)
+        self._build_and_set_new_pipeline()
+        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
         self.assertEqual(['counter_name'],
-                         pipeline.get_pipeline_grouping_key(
-                             pipeline_manager.pipelines[0]))
+                         pipeline_manager.pipelines[0].get_grouping_key())
